## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

## Author: Armin Auf der Maur <armin@armin-ENLE11BZ>N
## Created: 2020-05-20

## Randomization test. Permutations to test for H0
## R1dif2rr3: 1 = R, 2 = dif, 3 = rr (0 for hits in the tea tasting experiment) 
## 
function [retval] = H0regr(input1, input2, R1dif2rr3=2);
N = 10000, %for precision change to N = 100000,
  y = load(input1); % outcome 
  x = load(input2); % treatment
n = length(y);
switch (R1dif2rr3)
case (0) %number of hits in Fisher's tea tasting experiments
hits = 0;
for (i = 1:n),  if (y(i)-x(i) ==0), ++hits; endif, endfor, hits,
endswitch,
avy = mean(y);
sty = std(y, 1);
avy4 = ((y .- avy).^4);
B4 = mean(avy4)/sty.^4;
nB4 = n /B4, %effective sample size
xs = x(x != 0); ns = length(xs); %number of treated (seeded) days or cells
avx = mean(x);
stx = std(x, 1);
avxy = avx*avy;
stxy = sty*stx;
Rcr = avy*stx/(avx*sty), %singularity when calculating ratio rr by R
cdif = avx *sty *n /(stx *ns), %constant to calculate difference dif by R
R = (mean(y .* x) - avxy) / (sty * stx),
dif = R *cdif;
rr = 1 + R *(n / ns) /(-R + Rcr);
CorTie =-sign(R) /N;
Ri = zeros(1, N); hitsi = zeros(1, N);
switch (R1dif2rr3)
case (0)
for i = 1:N;
yrnd =y(randperm(n));
hj=0;
for (j = 1:n),  if (yrnd(j)-x(j) ==0); ++hj; endif, endfor,
hitsi(i) = hj; endfor
otherwise
for i = 1:N;
yrnd =y(randperm(n));
Ri(i) = (mean(x .* yrnd) - avxy)/stxy; endfor
endswitch

z =[1:N]; if(rem(N, 2)==0), z1=[1:N/2, N-[N/2:N-1]];
else, z1=[1:N/2+0.5, N-[0.5+N/2:N-1]]; endif,
Ri = sort(Ri); meanRi = mean(Ri), medRi = median(Ri)
switch (R1dif2rr3)
case (0)
hitsi = sort(hitsi);
Phits =lookup(hitsi, hits+CorTie)/N;
if(R ==0),Phits =(Phits +lookup(hitsi,hits-1/N)/N)/2; endif
Phits =min(Phits, 1 -Phits), 
semilogy(hitsi, z1/N, "b", hits, Phits, "xb")
xlabel("hits"); 
ylabel("P");
case (1)
PRH0 = lookup(Ri, R +CorTie)/N; %(R) takes care of ties
PRH0 =min(PRH0, 1-PRH0),
sigmaPRHO = ((PRH0 - PRH0 .^2)/N).^0.5,
semilogy(Ri, z1/N, "b", R, PRH0, "xb")
xlabel("R"), ylabel("P")
case (2)
dif = R *cdif
difi = Ri .*cdif;
Pdif = lookup(difi, dif +CorTie)/N;
Pdif =min(Pdif, 1-Pdif),
difL025 = difi(z == N*0.025)
difH975 = difi(z == (1+N*0.975))
semilogy(difi, z1/N, "b", dif, Pdif, "xb")
xlabel("dif (MJ)")
ylabel("P")
case (3), Rc1 =0.9*Rcr, Rc2 =-0.9*Rcr*(ns/(n-ns)), rri=zeros(1,N);
for (i = 1:N), rp(i)=max(min(Ri(i),Rc1),Rc2);
% Rc1 and Rc2 keep extreme permutations away from two singularities of rr.
rri(i)=max(min(1+(n/ns)/(-1+Rcr./rp(i)),99),0.011); endfor,
Prr = lookup(rri, rr +CorTie)/N;
Prr =min(Prr, 1-Prr),
loglog(rri, z1/N, "b", rr, Prr, "xb")
xlabel("rr")
ylabel("P")
endswitch
grid minor on
hold on
%res = [rr]; 
%res1 = res';
%save rrES.txt res1;
endfunction