## Copyright (C) 2020 Armin Auf der Maur
## 
## This program is free software; you can redistribute it and/or modify it
## under the terms of the GNU General Public License as published by
## the Free Software Foundation; either version 3 of the License, or
## (at your option) any later version.
## 
## This program is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
## GNU General Public License for more details.
## 
## You should have received a copy of the GNU General Public License
## along with this program.  If not, see <http://www.gnu.org/licenses/>.

## Author: Armin Auf der Maur <armin@armin-ENLE11BZ>
## Created: 2020-03-28
## pairwise (bivariate) bootstrap to calculate confidence intervals

function [retval] = H1Efron(input1,input2,R1dif2rr3=2);
NH0=10000; N=10000; PL=0.025, PU=0.025, %For P(H0), for CI, lower and upper 
y = load(input1); x = load(input2); n =length(y);

CorTie=0.01/N; %A small shift to catch the end of ties 

Sxn =(sum(x))/n; avy=mean(y); ns =(n-length(x(x ==0)));
R = (sum(x .* y)-Sxn*sum(y)) /(n *(std(x, 1) *std(y, 1))), % Pearsons corr coeff
dif = (sum(x .* y)-Sxn*sum(y))*Sxn /(var(x, 1) *ns), % difference (in MJ)
Rcr=avy*std(x, 1)/(mean(x)*std(y, 1)); rr=1+(n/ns)/(-1+Rcr/R),
nB4 = n /kurtosis(y), %effective sample size
P = 0; %First P(H0) is calculated by bootstrapping
xe =x; ns =(n-length(xe(xe ==0))); stx =std(xe, 1); mex=mean(xe);
for (i = 1:NH0)
ye=empirical_rnd(y);
%you may bootstrap y, x, or independently both
Rj = (mean(xe .* ye)-mean(ye)*mex) /(stx *std(ye, 1));

switch (R1dif2rr3), 
case (1),
if (Rj >= R), ++P; endif,
case (2)
difj = (sum(xe .* ye) -mex*sum(ye))*mex /((stx.^2) *ns);
if (difj >= dif), ++P; endif,
case (3),
Rc1 =0.9*Rcr; Rc2 =-0.9*Rcr*(ns/(n-ns));
rp=max(min(Rj,Rc1),Rc2);
rrj =max(min(1+(n/ns)/(-1+Rcr/rp),50),0.02);
if(rrj >= rr), ++P; endif
endswitch, endfor
PH0 = min(P/NH0, 1-P/NH0), sigmaPH0 = ((PH0 -PH0.^2)/NH0).^0.5,
Ri = zeros(1, N); difi = zeros(1, N); rri = zeros(1, N);

DATA = complex(y, x); %This enables pairwise (bivariate) bootstrapping
for (i = 1:N)
do
dabot = empirical_rnd(DATA);
x = imag(dabot); stx =std(x, 1);
y = real(dabot); sty =std(y, 1);
stxy =  stx *sty;
until (stxy >= CorTie)
ns =(n-length(xe(xe ==0)));mex =mean(x);
switch (R1dif2rr3)
case {1}
Ri(i) =(mean(x .* y)-mean(y)*mex) /stxy;
case {2}
difi(i) =(sum(x .* y)-mex*sum(y))*mex/((stx.^2)*ns);
case {3}
Ri = (mean(x .* y)-mean(y)*(mex)) /stxy;
Rcr=mean(y)*stx/(mex*sty); 
Rc1 =0.9*Rcr; Rc2 =-0.9*Rcr*(ns/(n-ns));
rp=max(min(Ri,Rc1),Rc2);
rri(i)=max(min(1+(n/ns)/(-1+Rcr/rp),50),0.02);
endswitch
endfor
z =[1:N]; if(rem(N, 2)==0), z1=[1:N/2, N-[N/2:N-1]];
else, z1=[1:N/2+0.5, N-[0.5+N/2:N-1]]; endif,
hold on
switch (R1dif2rr3)
case {1}
Ri = sort(Ri);
P0H1 = lookup(Ri, 0)/N;
if (P0H1 <= 0.5)
P0H1 = P0H1
else
P0H1 = 1 - P0H1
endif
sigmaP0H1 = sqrt((P0H1-P0H1.^2) /N)
meanR = mean(Ri)
medR = median(Ri)
LR = Ri(z == N*PL)
UR = Ri(z == N*(1-PU))
delt =R -medR, 
L16 = Ri(z == N*0.159) +delt
semilogy(Ri, z1/N, 'm', R, PH0, 'xm')%, L16, 0.159, 'sm', LR, PL, 'o', UR, PU, 'o'
xlabel ("R")

case {2}
difi = sort(difi);
P0H1 = lookup(difi, 0)/N;
if (P0H1 <= 0.5), P0H1, else, P0H1 = 1 - P0H1, endif, 
sigmaP0H1 = sqrt((P0H1-P0H1.^2) /N),
Ldif = difi(z == N*PL), Udif = difi(z == N*(1-PU)),
if (P0H1 <= 0.5), P0H1, else, P0H1 = 1 - P0H1, endif,
dif16=difi(z==N*0.159), sigmaP0H1 = sqrt((P0H1-P0H1.^2) /N);
%semilogy(difi,z1/N,".b",0,P0H1,"xb",dif, PH0,"xb",Ldif,PL,"ob",Udif,PU,"ob",dif16,0.16,"sb"),
%semilogy(difi,z1/N,".k", 0,P0H1,"xk",dif,PH0,"xk",Ldif,PL,"ok",Udif,PU,"ok",dif16,0.16,"sk"),
%semilogy(difi,z1/N,".r",0,P0H1,"xr",dif,PH0,"xr",Ldif,PL,"or",Udif,PU,"or",dif16,0.16,"sr"),
semilogy(difi,z1/N,"m",dif,PH0,"xm")%,0,P0H1,"xm",Ldif,PL,"om",Udif,PU,"om",dif16,0.16,"sm"),
xlabel("dif (MJ/cell)"),

case (3)
rri = sort(rri);
rrL025 = rri(z == N*PL)
rrH975 = rri(z == N*(1-PU))
meanrr = mean(rri); medrr = median(rri),
if (R>=0), rr16=rri(z==N*0.159), else, rr16=rri(z==N*0.841), endif
loglog(rri, z1/N, "r", rr, PH0, "xr", rr16, 0.159, "or")
%semilogy(rri, z1/N, '.', 1, P1H1, 'x', rri, PH0, 'x', rrL025, PL, 'o', rrH975, PU, 'o')
xlabel ("rr")

endswitch
ylabel ("P"), grid minor on
%res = [rr]; 
%res1 = res';
%save rrES.txt res1;
endfunction